'use strict';

const consola = require('consola');
const chalk = require('chalk');
const path = require('path');
const fs = require('fs');
const ora = require('ora');
const http = require('http');
const serveStatic = require('serve-static');
const core = require('@bubblewrap/core');
const fs$1 = require('fs-extra');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

const consola__default = /*#__PURE__*/_interopDefaultLegacy(consola);
const chalk__default = /*#__PURE__*/_interopDefaultLegacy(chalk);
const path__default = /*#__PURE__*/_interopDefaultLegacy(path);
const fs__default = /*#__PURE__*/_interopDefaultLegacy(fs);
const ora__default = /*#__PURE__*/_interopDefaultLegacy(ora);
const http__default = /*#__PURE__*/_interopDefaultLegacy(http);
const serveStatic__default = /*#__PURE__*/_interopDefaultLegacy(serveStatic);
const fs__default$1 = /*#__PURE__*/_interopDefaultLegacy(fs$1);

var name = "nuxt-twa";
var version = "0.0.0";

async function buildAab(context) {
  const progressLogger = ora__default['default']("Building AAB...").start();
  await context.tooling.gradleWrapper.bundleRelease();
  const unsignedAabPath = path__default['default'].resolve(context.options.targetDir, "./app/build/outputs/bundle/release/app-release.aab");
  if (context.keyConfig) {
    progressLogger.text = "Signing AAB...";
    const signedAabPath = path__default['default'].resolve(context.options.targetDir, "./app-release-bundle.aab");
    await context.tooling.jarSigner.sign(context.keyConfig, context.keyConfig.password, context.keyConfig.keypassword, unsignedAabPath, signedAabPath);
  } else {
    await fs__default['default'].promises.copyFile(unsignedAabPath, context.options.targetDir);
  }
  progressLogger.stop();
  consola__default['default'].success(`AAB built ${context.keyConfig ? "and" : chalk__default['default'].bold.yellow("NOT")} signed`);
}

async function buildApk(context) {
  const progressLogger = ora__default['default']("Building APK...").start();
  await context.tooling.gradleWrapper.assembleRelease();
  const unsignedApkPath = path__default['default'].resolve(context.options.targetDir, "./app/build/outputs/apk/release/app-release-unsigned.apk");
  if (context.keyConfig) {
    progressLogger.text = "Signing APK...";
    const signedApkPath = path__default['default'].resolve(context.options.targetDir, "./app-release-signed.apk");
    await context.tooling.androidSdkTools.apksigner(context.keyConfig.path, context.keyConfig.password, context.keyConfig.alias, context.keyConfig.keypassword, unsignedApkPath, signedApkPath);
  } else {
    await fs__default['default'].promises.copyFile(unsignedApkPath, context.options.targetDir);
  }
  progressLogger.stop();
  consola__default['default'].success(`APK built ${context.keyConfig ? "and" : chalk__default['default'].bold.yellow("NOT")} signed`);
}

async function generateAndroidProject(context) {
  const server = http__default['default'].createServer((req, res) => serveStatic__default['default'](context.nuxtOptions.generate.dir)(req, res, null)).listen(3e3);
  await new core.TwaGenerator().createTwaProject(context.options.targetDir, context.twaManifest, null);
  server.close();
  await fs__default['default'].promises.copyFile(path__default['default'].resolve(__dirname, "../assets/Android.gitignore"), path__default['default'].resolve(context.options.targetDir, ".gitignore"));
  consola__default['default'].success("Android project generated");
}

async function generateAssetLinks(context) {
  if (!context.keyConfig) {
    return consola__default['default'].warn("No Asset Links file generated because keys are not configured.");
  }
  const digitalAssetLinks = core.DigitalAssetLinks.generateAssetLinks(context.twaManifest.packageId, (await context.tooling.keyTool.keyInfo(context.keyConfig)).fingerprints.get("SHA256"));
  const linksPath = path__default['default'].resolve(context.nuxtOptions.generate.dir, ".well-known/assetlinks.json");
  await fs__default$1['default'].mkdirp(path__default['default'].dirname(linksPath), null);
  await fs__default$1['default'].promises.writeFile(linksPath, digitalAssetLinks);
  consola__default['default'].success("Asset links emitted");
}

async function generateTwaManifest(context) {
  const twaManifest = await core.TwaManifest.fromWebManifestJson(new URL("http://localhost:3000" + context.nuxtOptions.pwa._manifestMeta.href), context.nuxtOptions.manifest);
  twaManifest.host = context.options.host;
  twaManifest.startUrl = context.nuxtOptions.manifest.start_url;
  twaManifest.packageId = context.options.packageId || core.util.generatePackageId(context.options.host);
  context.twaManifest = twaManifest;
}

async function getKeyConfig(context) {
  const keyStorePath = process.env.KEYSTORE_PATH;
  const keyStorePassword = process.env.KEYSTORE_PASSWORD;
  const keyStoreKey = process.env.KEYSTORE_KEY;
  const keyStoreKeyPassword = process.env.KEYSTORE_KEY_PASSWORD;
  if (!keyStorePath || !keyStorePassword || !keyStoreKey || !keyStoreKeyPassword) {
    consola__default['default'].warn("Missing or invalid key store config, Android build will NOT be signed and no asset links will be generated.");
  } else {
    context.keyConfig = {
      path: keyStorePath,
      alias: keyStoreKey,
      keypassword: keyStoreKeyPassword,
      password: keyStorePassword
    };
  }
}

async function setupTooling(context) {
  const androidSdkPath = context.options.androidSdkPath || process.env.ANDROID_HOME || process.env.ANDROID_SDK_ROOT;
  if (!androidSdkPath) {
    throw new Error("No android SDK path found. Aborting Android generation");
  }
  const jdkPath = (context.options.jdkPath || process.env.JAVA_HOME || "").replace(/\/Contents\/Home\/?/, "");
  if (!jdkPath) {
    throw new Error("No Java JDK path found. Aborting Android generation");
  }
  const config = new core.Config(jdkPath, androidSdkPath);
  const jdkHelper = new core.JdkHelper(process, config);
  const androidSdkTools = await core.AndroidSdkTools.create(process, config, jdkHelper);
  const gradleWrapper = new core.GradleWrapper(process, androidSdkTools, context.options.targetDir);
  const keyTool = new core.KeyTool(jdkHelper);
  const jarSigner = new core.JarSigner(jdkHelper);
  context.tooling = {
    androidSdkTools,
    gradleWrapper,
    jarSigner,
    keyTool
  };
}

const tasks = {
  buildAab,
  buildApk,
  generateAndroidProject,
  generateAssetLinks,
  generateTwaManifest,
  getKeyConfig,
  setupTooling
};

function buildContext(moduleOptions, nuxtOptions) {
  return {
    options: {
      ...{targetDir: path__default['default'].join(nuxtOptions.srcDir, "android")},
      ...moduleOptions
    },
    nuxtOptions
  };
}

async function twa(moduleOptions) {
  const options = {
    ...this.options.twa,
    ...moduleOptions
  };
  const context = buildContext(options, this.nuxt.options);
  try {
    await Promise.all([
      tasks.getKeyConfig(context),
      tasks.generateTwaManifest(context),
      tasks.setupTooling(context)
    ]);
  } catch (e) {
    return consola__default['default'].error(e);
  }
  this.nuxt.hook("generate:done", async (_) => {
    consola__default['default'].info(`Starting TWA generation
      	packageId: ${chalk__default['default'].bold.green(context.twaManifest.packageId)}
      	host: ${chalk__default['default'].bold.green(context.twaManifest.host)}`);
    await tasks.generateAndroidProject(context);
    try {
      await Promise.all([
        tasks.buildApk(context),
        tasks.buildAab(context)
      ]);
    } catch (e) {
      consola__default['default'].error(e.stdout ? e.stdout : e);
    }
    await tasks.generateAssetLinks(context);
  });
}
twa.meta = {
  name,
  version
};

module.exports = twa;
